pub enum InvertibleComponentIdSet {
Included(ComponentIdSet),
Excluded(ComponentIdSet),
}Expand description
A set of bits that is either a finite set, or the set complement of a finite set.
Variants
Included(ComponentIdSet)
A finite InvertibleComponentIdSet that includes all components in the inner set.
Excluded(ComponentIdSet)
An unbounded InvertibleComponentIdSet that includes all components not in the inner set.
Implementations
Sourceimpl InvertibleComponentIdSet
impl InvertibleComponentIdSet
Sourcepub fn insert(&mut self, index: ComponentId)
pub fn insert(&mut self, index: ComponentId)
Adds a ComponentId to the set.
Sourcepub fn remove(&mut self, index: ComponentId)
pub fn remove(&mut self, index: ComponentId)
Removes a ComponentId from the set.
Sourcepub fn clear(&mut self)
pub fn clear(&mut self)
Removes all ComponentIds from the set.
Sourcepub fn all(&mut self)
pub fn all(&mut self)
Adds all ComponentIds to the set.
Sourcepub fn contains(&self, index: ComponentId) -> bool
pub fn contains(&self, index: ComponentId) -> bool
Returns true if the ComponentId is in the set.
Sourcepub fn is_clear(&self) -> bool
pub fn is_clear(&self) -> bool
Returns true if the set is empty.
Note that an unbounded set is never clear.
Sourcepub fn is_all(&self) -> bool
pub fn is_all(&self) -> bool
Returns true if the set contains all components.
Note that a finite set is never fully set.
Sourcepub fn is_unbounded(&self) -> bool
pub fn is_unbounded(&self) -> bool
Returns true if this is an unbounded set, or false if it is a finite set.
Sourcepub fn into_finite_set(self) -> Option<ComponentIdSet>
pub fn into_finite_set(self) -> Option<ComponentIdSet>
If this is a finite set, returns the set.
If this is an unbounded set, returns None.
Sourcepub fn as_finite_set(&self) -> Option<&ComponentIdSet>
pub fn as_finite_set(&self) -> Option<&ComponentIdSet>
If this is a finite set, returns the set.
If this is an unbounded set, returns None.
Sourcepub fn as_exclusion_set(&self) -> Option<&ComponentIdSet>
pub fn as_exclusion_set(&self) -> Option<&ComponentIdSet>
If this is an unbounded set, returns the set of components
that are excluded from the set.
If this is a finite set, returns None.
Sourcepub fn union_with(&mut self, other: &Self)
pub fn union_with(&mut self, other: &Self)
In-place union of two sets.
Sourcepub fn difference_with(&mut self, other: &Self)
pub fn difference_with(&mut self, other: &Self)
In-place difference of two sets.
Sourcepub fn difference(&self, other: &Self) -> Self
pub fn difference(&self, other: &Self) -> Self
Returns the set difference of two sets.
Sourcepub fn intersect_with(&mut self, other: &Self)
pub fn intersect_with(&mut self, other: &Self)
In-place intersection of two sets.
Sourcepub fn intersection(&self, other: &Self) -> Self
pub fn intersection(&self, other: &Self) -> Self
Returns the intersection of two sets.
Sourcepub fn is_disjoint(&self, other: &Self) -> bool
pub fn is_disjoint(&self, other: &Self) -> bool
Returns true if self has no elements in common with other.
This is equivalent to checking for an empty intersection.