QueryContiguousParIter

Struct QueryContiguousParIter 

pub struct QueryContiguousParIter<'w, 's, D, F>{ /* private fields */ }
A parallel iterator over contiguous query results on a Query.

The Query::contiguous_par_iter and Query::contiguous_par_iter_mut methods create instances of this structure.

Implementations

Source

impl<'w, 's, D, F> QueryContiguousParIter<'w, 's, D, F>

pub fn batching_strategy(self, strategy: BatchingStrategy) -> Self

Changes the batching strategy used when iterating.

For more information on how this affects the resultant iteration, see BatchingStrategy.

pub fn for_each( self, func: impl Fn(D::Contiguous<'w, 's>) + Send + Sync + Clone, )

Runs func on each contiguous chunk of query results in parallel.

Panics

If the ComputeTaskPool is not initialized. If using this from a query that is being initialized and run from the ECS scheduler, this should never panic.

pub fn for_each_init<T>( self, init: impl Fn() -> T + Sync + Send + Clone, func: impl Fn(&mut T, D::Contiguous<'w, 's>) + Send + Sync + Clone, )

Runs func on each query result in parallel on a value returned by init.

init may be called multiple times per thread, and the values returned may be discarded between tasks on any given thread. Callers should avoid using this function as if it were a parallel version of Iterator::fold.

Example
use bevy_utils::Parallel;
use crate::{bevy_ecs::prelude::Component, bevy_ecs::system::Query};
#[derive(Component)]
struct T;
fn system(query: Query<&T>){
    let mut queue: Parallel<usize> = Parallel::default();
    // queue.borrow_local_mut() will get or create a thread_local queue for each task/thread.
    // We unwrap the call to `contiguous_par_iter()` because we know the query in question is dense.
    query.contiguous_par_iter().unwrap().for_each_init(|| queue.borrow_local_mut(),|local_queue, items| {
        for _ in items {
            **local_queue += 1;
        }
     });

    // collect value from every thread
    let entity_count: usize = queue.iter_mut().map(|v| *v).sum();
}
Panics

If the ComputeTaskPool is not initialized. If using this from a query that is being initialized and run from the ECS scheduler, this should never panic.